import mimetypes
import datetime as dt

from ckan.logic import get_action

from ckanext.datapackage_creator.backends import default as inference_backend
from ckanext.datapackage_creator.model import Datapackage, DatapackageResource
from ckanext.datapackage_creator.converter import ckan_to_frictionless


def inference_data(context, data):
    filepath = data['filepath']
    result = inference_backend.describe_resource(filepath)
    try:
        rows = inference_backend.extract_resource(filepath)
        for field in result['schema']['fields']:
            field['rows'] = [row[field['name']] for row in rows]
            field['required'] = False
            field['description'] = ''
            field['format'] = 'default'
            field['unique'] = False
            field['extras'] = []
            field['primary_key'] = False
            field['foreign_key'] = ''
    except TypeError:
        pass
    data = {
        'metadata': result,
        'content_type': mimetypes.guess_type(filepath),
    }
    return data


def save_datapackage(context, data):
    metadata = data['metadata']
    package_id = data['package_id']
    Session = context['model'].Session
    datapackage = Datapackage()
    datapackage.package_id = package_id
    datapackage.data = metadata
    datapackage.created = dt.datetime.utcnow()
    datapackage.errors = data.get('errors')
    Session.add(datapackage)
    Session.commit()
    return datapackage


def save_datapackage_resource(context, data):
    metadata = data['metadata']
    resource_id = data['resource_id']
    Session = context['model'].Session
    datapackage_resource = DatapackageResource()
    datapackage_resource.resource_id = resource_id
    datapackage_resource.data = metadata
    datapackage_resource.created = dt.datetime.utcnow()
    datapackage_resource.errors = data.get('errors')
    Session.add(datapackage_resource)
    Session.commit()
    return datapackage_resource


def datapackage_show(context, data):
    package_id = data['package_id']
    Session = context['model'].Session
    datapackage = Session.query(Datapackage).filter(
        Datapackage.package_id==package_id
    ).order_by(Datapackage.created.desc()).first()
    return datapackage


def datapackage_resource_show(context, data):
    resource_id = data['resource_id']
    Session = context['model'].Session
    datapackage_resource = Session.query(DatapackageResource).filter(
        DatapackageResource.resource_id==resource_id
    ).order_by(DatapackageResource.created.desc()).first()
    return datapackage_resource


def generate_datapackage_json(context, data):
    package = get_action('package_show')(context, data)
    return ckan_to_frictionless(package)



import json
import pytest
import responses

import ckan.tests.factories as factories
import ckan.tests.helpers as helpers
import ckan.plugins.toolkit as toolkit


@pytest.mark.ckan_config('ckan.plugins', 'datapackage_creator')
@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestDataackageCreatorController():

    @responses.activate
    def test_inference_resource(self, app):
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        data = {
            'file': (open('test_data.csv', 'rb'), 'test_data.csv')
        }
        url = toolkit.url_for('datapackage_creator.inference')
        response = app.post(
            url, extra_environ=env, data=data
        )
        assert 200 == response.status_code
        result = json.loads(response.body)
        fields = ['id_fonte', 'cd_fonte', 'nome']
        result_fields = [f['name'] for f in result['metadata']['schema']['fields']]
        for field in fields:
            assert field in result_fields



import ckan.plugins as plugins

from ckan.model import Session

from flask import Blueprint

from ckanext.datapackage_creator.controllers import datapackage_creator
from ckanext.datapackage_creator.logic import (
    save_datapackage, inference_data, save_datapackage_resource, datapackage_show,
    datapackage_resource_show, generate_datapackage_json
)
from ckanext.datapackage_creator.converter import ckan_resource_to_frictionless
from ckanext.datapackage_creator.cli import get_commands
from ckanext.datapackage_creator.model import DatapackageResource
from ckanext.datapackage_creator.converter import ckan_to_frictionless


class DatapackageCreatorPlugin(plugins.SingletonPlugin):

    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IClick)
    # plugins.implements(plugins.IPackageController)
    # plugins.implements(plugins.IResourceController)

    # def after_dataset_show(self, context, pkg_dict):
    #     pkg_dict['datapackage_json'] = ckan_to_frictionless(pkg_dict)

    # def before_resource_update(self, *args, **kwargs):
    #     pass

    # def after_resource_update(self, *args, **kwargs):
    #     pass

    # def before_resource_show(self, resource_dict):
    #     resource_dict['table_schema'] = ckan_resource_to_frictionless(resource_dict)
    #     return resource_dict

    def get_blueprint(self):
        blueprint = Blueprint('datapackage_creator', __name__, url_prefix='/datapackage-creator')
        blueprint.add_url_rule(
            "/inference", view_func=datapackage_creator.inference,
            endpoint='inference', methods=['POST']
        )
        blueprint.add_url_rule(
            "/save-resource", view_func=datapackage_creator.save_resource,
            endpoint='save_resource', methods=['POST']
        )
        blueprint.add_url_rule(
            "/delete-resource/<resource_id>", view_func=datapackage_creator.delete_resource,
            endpoint='delete_resource', methods=['DELETE']
        )
        blueprint.add_url_rule(
            "/save-package", view_func=datapackage_creator.save_package,
            endpoint='save_package', methods=['POST']
        )
        blueprint.add_url_rule(
            "/publish-package", view_func=datapackage_creator.publish_package,
            endpoint='publish_package', methods=['POST']
        )
        blueprint.add_url_rule(
            "/show-datapackage/<package_id>", view_func=datapackage_creator.datapackage_show,
            endpoint='datapackage_show', methods=['GET']
        )
        blueprint.add_url_rule(
            "/show-datapackage-resource/<resource_id>", view_func=datapackage_creator.datapackage_resource_show,
            endpoint='datapackage_resource_show', methods=['GET']
        )
        blueprint.add_url_rule(
            "/show-datapackage-json/<package_id>", view_func=datapackage_creator.datapackage_json_show,
            endpoint='datapackage_json_show', methods=['GET']
        )
        blueprint.add_url_rule(
            "/show-validation/<package_id>", view_func=datapackage_creator.datapackage_validation_show,
            endpoint='datapackage_validation_show', methods=['GET']
        )
        blueprint.add_url_rule(
            "/show-settings", view_func=datapackage_creator.settings_show,
            endpoint='settings_show', methods=['GET']
        )
        return blueprint

    def update_config(self, config):
        plugins.toolkit.add_public_directory(config, 'public')
        plugins.toolkit.add_template_directory(config, 'templates')

    def get_actions(self):
        return {
            'save_datapackage': save_datapackage,
            'save_datapackage_resource': save_datapackage_resource,
            'inference_data': inference_data,
            'datapackage_show': datapackage_show,
            'datapackage_resource_show': datapackage_resource_show,
            'generate_datapackage_json': generate_datapackage_json,
        }

    def get_commands(self):
        return get_commands()



import click

from ckanext.datapackage_creator.model import create_tables


@click.command(u"datapackage-creator-init-db")
def init_db():
    """Create tables command.
    """
    create_tables()


def get_commands():
    return [init_db]



import os
import io

from setuptools import setup, find_packages


def read(*paths):
    """Read a text file."""
    basedir = os.path.dirname(__file__)
    fullpath = os.path.join(basedir, *paths)
    contents = io.open(fullpath, encoding="utf-8").read().strip()
    return contents


README = read("README.md")


setup(
    name='''ckanext-datapackage-creator''',
    version='0.0.39',
    description='''Data Package Creator.''',
    long_description=README,
    long_description_content_type="text/markdown",
    author='''Transparência Minas Gerais''',
    license='AGPL',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords='''CKAN''',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    namespace_packages=['ckanext'],
    install_requires=[
        'frictionless==4.40.8',
        'jsonschema',
        'python-slugify',
        'frictionless_ckan_mapper',
        'openpyxl',
        'xlrd',
    ],
    include_package_data=True,
    package_data={
    },
    data_files=[],
    entry_points={
        'ckan.plugins': [
            'datapackage_creator=ckanext.datapackage_creator.plugin:DatapackageCreatorPlugin'
        ],
    },
)


import json

from ckan.common import config

from ckanext.datapackage_creator.decorators import singleton


@singleton
class Settings(object):

    def __init__(self):
        self.config = {}
        datapackage_creator_config = config.get('datapackage_creator', None)
        if datapackage_creator_config is not None:
            with open(datapackage_creator_config, 'r') as datapackage_creator_json:
                self.config = json.loads(datapackage_creator_json.read())

    def get(self, name, default=None):
        return self.config.get(name, default)


settings = Settings()
